"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCoercers = createCoercers;

var _utils = require("./utils");

function createCoercers({
  initialValues,
  coerceRef,
  skin,
  onSubmit,
  schema
}) {
  return function coercedSubmit(doc) {
    const coerceObject = ({
      object,
      schemaDef
    }) => {
      const fields = Object.keys(schemaDef);
      const result = (0, _utils.deepmerge)({}, object);
      fields.forEach(fieldName => {
        const fieldSchema = schemaDef[fieldName];
        const {
          type
        } = fieldSchema;
        const typeKey = (0, _utils.schemaTypeEx)(type);
        const {
          coerce
        } = fieldSchema.coerce ? fieldSchema : skin[typeKey];
        const value = object[fieldName];

        if (coerce) {
          result[fieldName] = coerce(value, {
            coerceObject,
            schemaDef,
            fieldName
          });
        }
      });
      return result;
    };

    const coerceWithSchema = ({
      doc,
      schema
    }) => {
      const schemaDef = schema.getSchema();
      return coerceObject({
        object: doc,
        schemaDef
      });
    };

    const wholeObj = (0, _utils.deepmerge)({}, initialValues, doc, coerceRef.current);
    const coerced = coerceWithSchema({
      doc: wholeObj,
      schema
    });
    onSubmit(coerced, doc);
  };
}