"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AutoformBase = void 0;

var _react = _interopRequireWildcard(require("react"));

var _utils = require("../utils");

var _reactHookForm = require("react-hook-form");

var _componentRender = require("./componentRender");

var _coercing = require("../coercing");

var _baseSkin = _interopRequireDefault(require("./baseSkin"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

/**
 * Creates a form using the provided skin. The form
 * has all the needed fields, styles and validation
 * errors in order to work.
 */
let AutoformBase = (props, ref) => {
  const {
    schema,
    elementProps,
    initialValues = {},
    children,
    onSubmit,
    onErrors,
    styles,
    submitButton,
    submitButtonText,
    skin,
    skinOverride,
    skipManualReset
  } = props,
        rest = _objectWithoutProperties(props, ["schema", "elementProps", "initialValues", "children", "onSubmit", "onErrors", "styles", "submitButton", "submitButtonText", "skin", "skinOverride", "skipManualReset"]);

  if (!schema) {
    throw new Error('<Autoform /> was rendered without schema.');
  }

  const coerceRef = (0, _react.useRef)({});
  const initial = initialValues && (0, _utils.isObject)(initialValues) ? initialValues : {};
  const formHook = (0, _reactHookForm.useForm)({
    defaultValues: initial
  });
  const {
    formState,
    register,
    unregister,
    handleSubmit,
    errors,
    watch,
    reset,
    getValues
  } = formHook;

  const finalSkin = _objectSpread(_objectSpread(_objectSpread({}, _baseSkin.default), skin), skinOverride);

  const setValue = (name, value, options, skipSetInput) => {
    const [container, attr] = (0, _utils.objectTraverse)(coerceRef.current, name, {
      createIfMissing: true
    });
    if (container && attr) container[attr] = value;
    if (!skipSetInput) formHook.setValue(name, value, options);
  };

  if (onErrors && Object.keys(errors).length > 0) onErrors(errors);
  const coercedSubmit = (0, _coercing.createCoercers)({
    initialValues: initial,
    coerceRef,
    skin: finalSkin,
    onSubmit,
    schema
  });

  const resetAlsoCoercers = (values, omit) => {
    coerceRef.current = {};
    (0, _utils.deepmerge)(coerceRef.current, values || {});
    const currentValues = getValues();

    if (!skipManualReset) {
      // Reset by setting everything to initialValues or null.
      function resetValues(obj, initials = {}, path = '', isArray = false) {
        const fields = Object.keys(obj);
        fields.forEach((field, idx) => {
          const value = obj[field];
          const elPath = isArray ? `${path}[${idx}]` : path ? `${path}.${field}` : field;
          const initial = initials[field];
          if (typeof value == 'object') resetValues(value, initial, elPath, Array.isArray(value));else {
            const initialOrNull = typeof initial == 'undefined' ? null : initial;
            formHook.setValue(elPath, initialOrNull);
          }
        });
      }

      resetValues(currentValues, initialValues);
    }

    reset(values, omit);
  };

  const submit = handleSubmit(coercedSubmit);
  (0, _react.useImperativeHandle)(ref, () => ({
    submit,
    formHook: () => formHook,
    setValue,
    reset: resetAlsoCoercers
  }));

  const inputProps = _objectSpread(_objectSpread(_objectSpread({}, rest), elementProps), {}, {
    reset,
    setValue,
    children,
    initialValues: initial,
    schema,
    register,
    unregister,
    styles,
    errors,
    coerceRef,
    skin: finalSkin,
    formHook,
    autoformProps: props
  });

  const Button = finalSkin.button.render;
  const Form = finalSkin.form.render;
  return /*#__PURE__*/_react.default.createElement(Form, {
    onSubmit: submit
  }, (0, _componentRender.renderInputs)(inputProps), submitButton && /*#__PURE__*/_react.default.createElement(Button, {
    styles: styles,
    onClick: submit,
    type: "submit"
  }, submitButtonText), children);
};

exports.AutoformBase = AutoformBase;
exports.AutoformBase = AutoformBase = (0, _react.forwardRef)(AutoformBase);