"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validationRules = validationRules;
exports.registerWithRules = registerWithRules;
exports.registerValidation = registerValidation;
exports.renderInput = renderInput;
exports.renderInputs = renderInputs;

var _react = _interopRequireWildcard(require("react"));

var _classnames = _interopRequireDefault(require("classnames"));

var _utils = require("../utils");

var _translate = require("../translate");

var _translation_utils = require("../translation_utils");

var _FieldPropsOverride = require("./components/FieldPropsOverride");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const validations = {
  required: ({
    value,
    message
  }) => message,
  maxLength: 'maxLength',
  minLength: 'minLength',
  max: 'max',
  min: 'min',
  pattern: 'pattern',
  validate: 'validate'
};
/**
 * Creates validation rules after schema
 *
 * @param {object} fieldSchema
 */

function validationRules(fieldSchema) {
  const validationKeys = Object.keys(validations);
  return validationKeys.reduce((result, key) => {
    if (key in fieldSchema) {
      const validation = fieldSchema[key];
      let data;

      if (typeof validation == 'object') {
        if (validation.message && typeof validation.message == 'function') validation.message = validation.message(fieldSchema);
        data = validation;
      } else {
        data = {
          value: fieldSchema[key],
          message: (0, _translation_utils.trError)(key, fieldSchema)
        };
      }

      result[key] = typeof validations[key] == 'function' ? validations[key](data) : data;
    }

    return result;
  }, {});
}
/**
 * HoC for register that automatically adds validation rules
 *
 * @param {object} rules ReactHookForm rules
 * @param {function} register useForm's register
 */


function registerWithRules(rules, register) {
  if (Object.keys(rules).length == 0) return register;else return params => params ? register(params, rules) : register(rules);
}
/**
 * Passes the validation parameters to react-hook-form
 *
 * @param {object} fieldSchema Schema for the field
 * @param {function} register react-hook-form register
 */


function registerValidation(fieldSchema, register) {
  const rules = validationRules(fieldSchema);
  return registerWithRules(rules, register);
}
/**
 * Searches in children to find overrides.
 */


function searchForOverrides(parent, name, children = []) {
  const childrenArr = _react.Children.map(children, child => child);

  return childrenArr.reduce((override, child) => {
    const childName = child.props.name;
    const isOverride = child.type == _FieldPropsOverride.FieldPropsOverride;
    const unbracked = name.replace(/ *\[[^)]*\] */g, '');

    if (isOverride && (childName == name || unbracked == name)) {
      const cloned = Object.assign({}, child.props);
      delete cloned.name;
      return cloned;
    } else {
      return override;
    }
  }, {});
}

function renderSingleInput(props) {
  const {
    component,
    wrapper,
    name,
    field,
    type,
    defaultValue,
    option,
    inline,
    register,
    key,
    styles,
    fieldSchema,
    noRef,
    errors = {},
    noAutocomplete
  } = props,
        rest = _objectWithoutProperties(props, ["component", "wrapper", "name", "field", "type", "defaultValue", "option", "inline", "register", "key", "styles", "fieldSchema", "noRef", "errors", "noAutocomplete"]);

  const actualKey = option ? `${key}.${option}` : key;
  const $wrapper = wrapper;
  const $component = component;
  const isComponent = typeof component != 'string';

  const baseProps = _objectSpread({
    key: name,
    name: name,
    type: type,
    value: option,
    defaultValue,
    className: (0, _classnames.default)(styles.input, styles.standard, {
      [styles.errored]: errors[field]
    })
  }, fieldSchema.addInputProps);

  if (!isComponent && !noRef) baseProps.ref = register;
  const componentProps = isComponent ? _objectSpread(_objectSpread(_objectSpread({}, baseProps), rest), {}, {
    field,
    errors,
    fieldSchema,
    styles,
    register
  }) : baseProps;
  if (noAutocomplete || fieldSchema.noAutocomplete) componentProps.autoComplete = 'off';
  return /*#__PURE__*/_react.default.createElement($wrapper, _extends({
    key: actualKey,
    name: name,
    field: field,
    styles: styles,
    fieldSchema: fieldSchema,
    errors: errors,
    inline: inline,
    addWrapperProps: fieldSchema.addWrapperProps
  }, rest), /*#__PURE__*/_react.default.createElement($component, componentProps));
}
/**
 * Renders a single field.
 *
 * @param {object} params
 * @param {string} params.field Name of the field
 * @param {object} params.fieldSchema Schema specification
 *    for the field
 * @param {string} params.parent Prefix of the field name
 * @param {string} params.schemaTypeName Name of the schema
 *    (first argument while instantiating a schema)
 * @param {object} params.config Form configuration
 * @param {...object} params.rest props passed to the component
 */


function renderInput(_ref) {
  let {
    field,
    fieldSchema,
    fieldSchema: {
      type,
      required,
      defaultValue
    },
    initialValue,
    register,
    parent,
    children,
    propOverrides,
    schemaTypeName,
    config = {},
    index,
    errors,
    skin,
    styles
  } = _ref,
      rest = _objectWithoutProperties(_ref, ["field", "fieldSchema", "fieldSchema", "initialValue", "register", "parent", "children", "propOverrides", "schemaTypeName", "config", "index", "errors", "skin", "styles"]);

  const strType = (0, _utils.schemaTypeEx)(type);

  function describePlace() {
    return `Schema "${schemaTypeName}" has field "${field}"`;
  }

  if (!strType) {
    throw `${describePlace()} that lacks type description.`;
  }

  const skinElement = skin[strType];

  if (!skinElement) {
    throw `${describePlace()} with type "${strType}" ` + 'that doesn\'t exist in skin.';
  }

  const {
    render,
    wrapper
  } = skinElement;
  const rules = validationRules(fieldSchema);
  const validatedRegister = registerWithRules(rules, register);

  if (render) {
    let fullField;
    if (typeof index == 'undefined') fullField = parent ? `${parent}.${field}` : field;else fullField = `${parent || ''}[${index}].${field}`;
    const overrides = searchForOverrides(parent, fullField, propOverrides);

    const baseProps = _objectSpread(_objectSpread({}, rest), {}, {
      key: fullField,
      name: fullField,
      field,
      fieldSchema,
      schemaTypeName,
      config,
      parent,
      propOverrides,
      wrapper: wrapper || skin.defaultWrap,
      register: validatedRegister,
      rules,
      styles,
      skin,
      errors,
      defaultValue: initialValue !== null && initialValue !== void 0 ? initialValue : defaultValue,
      overrides
    }, overrides);

    let allProps;
    if (typeof render == 'function') allProps = render ? render(baseProps) : baseProps;else allProps = _objectSpread(_objectSpread({}, baseProps), render);
    if (Array.isArray(allProps)) return allProps.map(renderSingleInput);else return renderSingleInput(allProps);
  } else return null;
}
/**
 * Renders the inputs to make the schema work.
 *
 * @param {object} params
 * @param {Schema} params.schema Schema instance
 * @param {object} params.config Rendering configuration
 * @param {string} params.config.arrayMode 'panels' or 'table'
 * @param {...object} params.rest Props passed to every input
 *
 * @returns {array} React elements with the form and inputs.
 */


function renderInputs(_ref2) {
  let {
    schema,
    config = {},
    children,
    register,
    errors,
    propOverrides,
    initialValues = {},
    styles = {}
  } = _ref2,
      rest = _objectWithoutProperties(_ref2, ["schema", "config", "children", "register", "errors", "propOverrides", "initialValues", "styles"]);

  const schemaDef = schema.getSchema();
  const schemaKeys = Object.keys(schemaDef);
  return schemaKeys.map(field => renderInput(_objectSpread(_objectSpread({}, rest), {}, {
    register,
    field,
    config,
    errors,
    propOverrides: propOverrides || children,
    fieldSchema: schemaDef[field],
    schemaTypeName: schema.getType(),
    initialValue: initialValues[field],
    styles
  })));
}